/**********************************************************************

	--- Qt Architect generated file ---

	File: prefsDialog.h
	Last generated: Thu Jul 6 17:58:38 2000

 *********************************************************************/

#ifndef prefsDialog_included
#define prefsDialog_included

#include <kurllabel.h>
#include <qcheckbox.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qtoolbutton.h>
#include <qfont.h>
#include "prefdialogview.h"

class prefsDialog : public prefsDialogData
{
    Q_OBJECT

public:

    prefsDialog(QWidget *parent = 0,const char *name = "name" );

    virtual ~prefsDialog();
    /** Get the report location string. */
    QString reportLocation() const;
    QFont font() const;
    int updateInterval() const;
    bool offLineMode() const;
    bool smallView() const;
    bool enableLog() const;
    QString logFile() const;
    /** Set the pref box file name to the current prefs file name. */
    void setLogFile( QString s );
    /** Set the small view mode checkbox */
    void setSmallView( bool b);
    /** Set the LCD and Slider value */
    void setUpdateInterval(int i);
    /** Set the offline mode checkbox */
    void setOffLineMode( bool b );
    /** The current report location */
    void setReportLocation( QString s );
    void setEnableLog(bool);
    void setFont(QFont theFont);

private slots:
    /** Open KFM to this link. */
    void processURL(const QString&);
    /** Make new fonts **/
    void selectFont();
    // Enable/disable logfile input
    void enableLogWidgets(bool b);
   /** Offline mode was toggled **/
   void offlineModeToggled(bool);

private:
    QFont theFont;
    void updateFont();
};
#endif // prefsDialog_included
